<?php

function auto_discovery_link_tag($type = 'rss', $url_options = array(), $tag_options = array())
{
  return tag('link', array(
    'rel'   => isset($tag_options['rel']) ? $tag_options['rel'] : 'alternate',
    'type'  => isset($tag_options['type']) ? $tag_options['type'] : 'application/' . $type . '+xml',
    'title' => isset($tag_options['title']) ? $tag_options['title'] : ucfirst($type),
    'href'  => url_for($url_options, true)
  ));
}



function javascript_path($source, $absolute = false)
{
  return _compute_public_path($source, 'js', 'js', $absolute);
}


function javascript_include_tag()
{
  $html = '';
  foreach (func_get_args() as $source) {
    $source = javascript_path($source);
    $html .= content_tag('script', '', array('type' => 'text/javascript', 'src' => $source)) . "\n";
  }

  return $html;
}


function stylesheet_path($source, $absolute = false)
{
  return _compute_public_path($source, 'css', 'css', $absolute);
}

function http()
{
  if ((!empty($_SERVER['REQUEST_SCHEME']) && $_SERVER['REQUEST_SCHEME'] == 'https') ||
    (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ||
    (!empty($_SERVER['SERVER_PORT']) && $_SERVER['SERVER_PORT'] == '443')
  ) {
    return 'https://';
  } else {
    return 'http://';
  }
}

function locationUrl()
{
  return sfConfig::get('app_siteurl') . $_SERVER['REQUEST_URI'];
}

function stylesheet_tag()
{
  $sources = func_get_args();
  $sourceOptions = (func_num_args() > 1 && is_array($sources[func_num_args() - 1])) ? array_pop($sources) : array();

  $html = '';
  foreach ($sources as $source) {
    $source  = stylesheet_path($source);
    $options = array_merge(array('rel' => 'stylesheet', 'type' => 'text/css', 'media' => 'screen', 'href' => $source), $sourceOptions);
    $html   .= tag('link', $options) . "\n";
  }

  return $html;
}


function use_stylesheet($css, $position = '', $options = array())
{
  sfContext::getInstance()->getResponse()->addStylesheet($css, $position, $options);
}


function use_javascript($js, $position = '')
{
  sfContext::getInstance()->getResponse()->addJavascript($js, $position);
}


function decorate_with($layout)
{
  $view = sfContext::getInstance()->getActionStack()->getLastEntry()->getViewInstance();
  if (false === $layout) {
    $view->setDecorator(false);
  } else {
    $view->setDecoratorTemplate($layout);
  }
}


function image_path($source, $absolute = false)
{
  return _compute_public_path($source, 'images', 'png', $absolute);
}


function image_tag($source, $options = array())
{
  if (!$source) {
    return '';
  }

  $options = _parse_attributes($options);

  $absolute = false;
  if (isset($options['absolute'])) {
    unset($options['absolute']);
    $absolute = true;
  }

  $options['src'] = image_path($source, $absolute);
  if (!isset($options['alt'])) {
    $path_pos = strrpos($source, '/');
    $dot_pos = strrpos($source, '.');
    $begin = $path_pos ? $path_pos + 1 : 0;
    $nb_str = ($dot_pos ? $dot_pos : strlen($source)) - $begin;
    $options['alt'] = ucfirst(substr($source, $begin, $nb_str));
  }
  if (isset($options['size'])) {
    list($options['width'], $options['height']) = explode('x', $options['size'], 2);
    unset($options['size']);
  }
  elseif(!isset($options['size'])&&!isset($options['width'])&&!isset($options['height'])){
      list($width, $height) = getimagesize($_SERVER["DOCUMENT_ROOT"].'/'.$options['src']);
      $options['width'] = $width;
      $options['height'] = $height;
  }

  return tag('img', $options);
}

function _compute_public_path($source, $dir, $ext, $absolute = false)
{
  if (strpos($source, '://')) {
    return $source;
  }

  $request = sfContext::getInstance()->getRequest();
  $sf_relative_url_root = $request->getRelativeUrlRoot();
  if (0 !== strpos($source, '/')) {
    $source = $sf_relative_url_root . '/' . $dir . '/' . $source;
  }

  $query_string = '';
  if (false !== $pos = strpos($source, '?')) {
    $query_string = substr($source, $pos);
    $source = substr($source, 0, $pos);
  }

  if (false === strpos(basename($source), '.')) {
    $source .= '.' . $ext;
  }

  if ($sf_relative_url_root && 0 !== strpos($source, $sf_relative_url_root)) {
    $source = $sf_relative_url_root . $source;
  }

  if ($absolute) {
    $source = 'http' . ($request->isSecure() ? 's' : '') . '://' . $request->getHost() . $source;
  }

  return $source . $query_string;
}


function include_http_metas(){
    if($httpMetas=sfContext::getInstance()->getResponse()->getHttpMetas())
    foreach ($httpMetas as $httpequiv => $value)
    {
        if(strtolower($httpequiv)!="content-type"){echo tag('meta', array('http-equiv' => 'Content-Type', 'content' => 'text/html; charset=utf-8'))."\n";}
        if($value){
        if($value=="text/html"){echo tag('meta', array('http-equiv' => $httpequiv, 'content' => $value))."\n";}
        elseif(strtolower($httpequiv)=="language"){echo tag('meta', array('http-equiv' => 'content-language', 'content' => $value))."\n";}
        else {echo tag('meta', array('http-equiv' => $httpequiv, 'content' => $value))."\n";}
        }
    }
}

function getTitle(){
    $content = sfContext::getInstance()->getResponse()->getTitle();
    return $content;
}

function include_title()
{
    echo '<title>'.getTitle().'</title>'."\n";
}

function getCanonical(){
    $uri=urlencode($_SERVER['REQUEST_URI']);
    $canonicalURL=url_for(sfContext::getInstance()->getResponse()->getCanonical());
    if( !preg_match('/http(s?)\:\/\//i', $canonicalURL) ) {$canonicalURL=sfConfig::get('app_siteurl').$canonicalURL;}
    return $canonicalURL;
}

function include_metas(){
    foreach (sfContext::getInstance()->getResponse()->getMetas() as $name => $content)
    {
        if($name=='canonical'){
            $uri=urlencode($_SERVER['REQUEST_URI']);
            $canonicalURL=getCanonical();
            echo '<link rel="canonical" href="'.$canonicalURL.'" />'."\n";
            if(!preg_match("/$uri/",urlencode($canonicalURL)))
            echo '<link rel="alternate" href="'.sfConfig::get('app_siteurl').urldecode($uri).'" />'."\n";
        }
        elseif($name=='keywords'){
            echo '<meta name="keywords" content="'.$content.'" />'."\n";
        }
        elseif($name=='description'){
            echo '<meta name="description" content="'.$content.'" />'."\n";
        }
        elseif($name!='title'){echo tag('meta', array('name' => $name, 'content' => $content))."\n";}
  }
  
  if(!$description){$description=sfContext::getInstance()->getResponse()->getTitle();}
    echo tag('meta', array('name' => "og_title",      'property'=>'og:title',      'content' => sfContext::getInstance()->getResponse()->getTitle()))."\n";
    echo tag('meta', array('name' => "og_description",'property'=>'og:description','content' => $description))."\n";
    echo tag('meta', array('name' => "og_url",        'property'=>'og:url',        'content' => getCanonical()))."\n"; 
    echo tag('meta', array('name' => "og_site_name",  'property'=>'og:site_name',  'content' => sfConfig::get('app_sitename')))."\n";
    echo tag('meta', array('name' => "twitter:card",  'property'=>'twitter:card',  'content' => 'summary'))."\n";
    echo tag('meta', array('name' => "twitter:url",   'property'=>'twitter:url',   'content' => getCanonical()))."\n";
    echo tag('meta', array('name' => "twitter:site",  'property'=>'twitter:site',  'content' => '@'.sfConfig::get('app_smallname')))."\n";
    echo tag('meta', array('name' => "twitter:title", 'property'=>'twitter:title', 'content' => sfContext::getInstance()->getResponse()->getTitle()))."\n";
    echo tag('meta', array('name' => "twitter:description", 'property'=>'twitter:description', 'content' =>  $description))."\n";
}

function include_ogMeta($fileId, $catId)
{
  $thumbPath = sfConfig::get('sf_upload_dir') . "/thumb/";
  if ($fileId) {
    $thumbServer = 'sft' . ceil($fileId / 500);
    $catId = skyGetRecordById('files', $fileId)->category_id;
    if (is_file($thumbPath . '/' . $thumbServer . '/' . $fileId . '_6.jpg')) {
      $image = SETTING_THUMB_DOMAIN . '/' . $thumbServer . '/' . $fileId . '_6.jpg';
    } elseif (is_file($thumbPath . '/c/' . $catId . '_6.jpg')) {
      $image = SETTING_THUMB_DOMAIN . '/c/' . $catId . '_6.jpg';
    }
  } elseif ($catId) {
    if (is_file($thumbPath . '/c/' . $catId . '_6.jpg')) $image = SETTING_THUMB_DOMAIN . '/c/' . $catId . '_6.jpg';
  }
  if (!$image) {
    $image = '/images/android-chrome-512x512.png';
  }
  if ($image) {
    if (!preg_match('/http(s?)\:\/\//i', $image)) {
      $image = sfConfig::get('app_siteurl') . $image;
    }
    echo tag('meta', array('name' => 'og_image', 'property' => 'og:image', 'content' => $image)) . "\n";
    echo tag('meta', array('name' => 'msapplication-TileImage', 'content' => $image)) . "\n";
    echo tag('meta', array('property' => 'og:image:type', 'content' => 'image/jpeg')) . "\n";
    echo tag('meta', array('name' => 'twitter:image', 'property' => 'twitter:image', 'content' => $image)) . "\n";
  }
}


function getImage($fileId, $catId = "")
{
  $thumbPath = sfConfig::get('sf_upload_dir') . "/thumb/";
  if ($fileId) {
    $thumbServer = 'sft' . ceil($fileId / 500);
    $catId = skyGetRecordById('files', $fileId)->category_id;
    if (is_file($thumbPath . '/' . $thumbServer . '/' . $fileId . '_6.jpg')) {
      $image = SETTING_THUMB_DOMAIN . '/' . $thumbServer . '/' . $fileId . '_6.jpg';
    } elseif (is_file($thumbPath . '/c/' . $catId . '_6.jpg')) {
      $image = SETTING_THUMB_DOMAIN . '/c/' . $catId . '_6.jpg';
    }
  } elseif ($catId) {
    if (is_file($thumbPath . '/c/' . $catId . '_6.jpg')) $image = SETTING_THUMB_DOMAIN . '/c/' . $catId . '_6.jpg';
  }
  if (!$image) {
    $image = '/images/albumart.jpg';
  }
  if (!$fileId && !$catId) {
    $image = '/images/android-chrome-512x512.png';
  }
  if ($image) {
    if (!preg_match('/http(s?)\:\/\//i', $image)) {
      $image = sfConfig::get('app_siteurl') . $image;
    }
    return $image;
  }
}


function get_javascripts()
{
  $response = sfContext::getInstance()->getResponse();
  $response->setParameter('javascripts_included', true, 'symfony/view/asset');

  $already_seen = array();
  $html = '';

  foreach (array('first', '', 'last') as $position) {
    foreach ($response->getJavascripts($position) as $files) {
      if (!is_array($files)) {
        $files = array($files);
      }

      foreach ($files as $file) {
        $file = javascript_path($file);

        if (isset($already_seen[$file])) continue;

        $already_seen[$file] = 1;
        $html .= javascript_include_tag($file);
      }
    }
  }

  return $html;
}


function include_javascripts()
{
  echo get_javascripts();
}


function get_stylesheets()
{
  $response = sfContext::getInstance()->getResponse();
  $response->setParameter('stylesheets_included', true, 'symfony/view/asset');

  $already_seen = array();
  $html = '';

  foreach (array('first', '', 'last') as $position) {
    foreach ($response->getStylesheets($position) as $files => $options) {
      if (!is_array($files)) {
        $files = array($files);
      }

      foreach ($files as $file) {
        $file = stylesheet_path($file);

        if (isset($already_seen[$file])) continue;

        $already_seen[$file] = 1;
        $html .= stylesheet_tag($file, $options);
      }
    }
  }

  return $html;
}


function include_stylesheets()
{
  echo get_stylesheets();
}
